﻿using System;
using System.Collections.Generic;

namespace Eingabeverwaltung.Metamodell
{
	public class Lemma : Rule
	{
		public readonly List<Variable> fixedVars;
		public readonly SingleProof proof;
		private Nullable<bool> isProved;
		public readonly List<Lemma> cyclicDependencies;
		private SingleTask ownTask;
		public readonly List<TaskAssignment> usefulFor;
		public readonly List<Error> errors;

		public Lemma(Coords coords, string name, Equation equation, List<Variable> fixedVars, SingleProof proof) : base(coords, name,equation)
		{
			this.fixedVars = fixedVars;
			this.proof = proof;
			cyclicDependencies = null;
			errors = new List<Error>();
			isProved = null;
			cyclicDependencies = new List<Lemma>();
			ownTask = null;
			usefulFor = new List<TaskAssignment>();
		}

		public Nullable<bool> IsProved
        {
			get { return isProved; }
            set { if (isProved == null) isProved = value; else throw new KontextException(coords, "Tried to change existing isProved in Lemma!"); }
        }

		public SingleTask OwnTask
        {
			get { return ownTask; }
			set { if (ownTask == null) ownTask = value; else throw new KontextException(coords, "Tried to change existing ownTask in Lemma!"); }
		}

		/// <summary>
		/// Returns TaskAssignment ta from usefulFor with ta.task == task.
		/// Else returns null.
		/// </summary>
		/// <param name="task"></param>
		/// <returns></returns>
		public TaskAssignment GetTaskAssignment(SingleTask task) 
		{
			foreach (TaskAssignment ta in usefulFor)
				if (ta.task == task) return ta;
			return null;
		}

		public override bool IsFixed(Variable variable) => fixedVars.Contains(variable);

		public string ToString(bool showMetadata)
        {
			string res = "lemma " + base.ToString();
			try
			{
				if (showMetadata)
				{
					if (isProved != null) res += "isProved: " + ((bool)isProved) + "\r\n";
					if (cyclicDependencies.Count != 0)
					{
						res += "cylic Dependencies: ";
						foreach (Lemma lemma in cyclicDependencies) res += lemma.name + " ";
						res += "\r\n";
					}
					if (ownTask != null) res += "ownTask: " + ownTask.name + "\r\n";
					if (usefulFor.Count != 0)
					{
						res += "useful For: ";
						foreach (TaskAssignment ta in usefulFor)
							res += ta.ToString() + "  ";
						res += "\r\n";
					}
				}
				res += proof.ToString(showMetadata);
			}
			catch (NullReferenceException) { res += "NullReferenceException occured while Lemma.ToString."; }
			return res;
        }
    }
}
